@@ -4,12 +4,9 @@ namespace ParagonIE\ConstantTime\Tests;
 
 use ParagonIE\ConstantTime\Binary;
 
-/**
- * @method getNextChar(string $c): string
- */
 trait CanonicalTrait
 {
-    public function canonicalDataProvider(): array
+    public static function canonicalDataProvider(): array
     {
         return [
             ['a'],
@@ -21,6 +18,8 @@ trait CanonicalTrait
         ];
     }
 
+    abstract protected function getNextChar(string $c): string;
+
     protected function increment(string $str): string
     {
         $i = Binary::safeStrlen($str) - 1;
